@@ -1,7 +1,7 @@
 package Unix::Passwd::File;
 
-our $DATE = '2015-10-22'; # DATE
-our $VERSION = '0.22'; # VERSION
+our $DATE = '2016-10-07'; # DATE
+our $VERSION = '0.23'; # VERSION
 
 use 5.010001;
 use strict;
@@ -295,13 +295,17 @@ sub _routine {
     my $etc     = $args{etc_dir} // "/etc";
     my $detail  = $args{detail};
     my $wfn     = $args{with_field_names} // 1;
-    my $lock;
+    my @locks;
     my ($fhp, $fhs, $fhg, $fhgs);
     my %stash;
 
     my $e = eval {
 
-        $lock = File::Flock::Retry->lock("$etc/passwd.lock", {retries=>3});
+        if ($args{_lock}) {
+            for (qw/passwd shadow group gshadow/) {
+                push @locks, File::Flock::Retry->lock("$etc/$_", {retries=>3});
+            }
+        }
 
         # read files
 
@@ -486,7 +490,8 @@ sub _routine {
     }; # eval
     $e = [500, "Died: $@"] if $@;
 
-    undef $lock;
+    # release the locks
+    undef @locks;
 
     $stash{res} //= $e if $e && $e->[0] != 200;
     $stash{res} //= $e if $e && $e->[0] != 200;
@@ -546,9 +551,8 @@ sub list_users {
             }
 
             $stash->{res} = [200, "OK", \@rows];
-            $stash->{res}[3]{format_options}{any}{table_column_orders} =
-                [\@passwd_field_names] if $detail;
-
+            $stash->{res}[3]{'table.fields'} = [\@passwd_field_names]
+                if $detail;
             $stash->{exit}++;
             [200];
         },
@@ -707,9 +711,7 @@ sub list_groups {
             }
 
             $stash->{res} = [200, "OK", \@rows];
-            $stash->{res}[3]{format_options}{any}{table_column_orders} =
-                [\@group_field_names] if $detail;
-
+            $stash->{res}[3]{'table.fields'} = [\@group_field_names] if $detail;
             $stash->{exit}++;
             [200];
         },
@@ -1934,7 +1936,7 @@ Unix::Passwd::File - Manipulate /etc/{passwd,shadow,group,gshadow} entries
 
 =head1 VERSION
 
-This document describes version 0.22 of Unix::Passwd::File (from Perl distribution Unix-Passwd-File), released on 2015-10-22.
+This document describes version 0.23 of Unix::Passwd::File (from Perl distribution Unix-Passwd-File), released on 2016-10-07.
 
 =head1 SYNOPSIS
 
@@ -1998,7 +2000,7 @@ to search in custom location, for testing purposes).
 This module uses a procedural (non-OO) interface. Each function in this module
 open and read the passwd files once. Read-only functions like `list_users()` and
 `get_max_gid()` open in read-only mode. Functions that might write to the files
-like `add_user()` or `delete_group()` first lock `passwd.lock` file, open in
+like `add_user()` or `delete_group()` first lock `passwd` file, open in
 read+write mode and also read the files in the first pass, then seek to the
 beginning and write back the files.
 
@@ -2986,6 +2988,22 @@ Specify location of passwd files.
 
 Return value:  (bool)
 
+=head1 HOMEPAGE
+
+Please visit the project's homepage at L<https://metacpan.org/release/Unix-Passwd-File>.
+
+=head1 SOURCE
+
+Source repository is at L<https://github.com/perlancar/perl-Unix-Passwd-File>.
+
+=head1 BUGS
+
+Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Unix-Passwd-File>
+
+When submitting a bug or request, please include a test-file or a
+patch to an existing test-file that illustrates the bug or desired
+feature.
+
 =head1 SEE ALSO
 
 Old modules on CPAN which do not support shadow files are pretty useless to me
@@ -3021,7 +3039,7 @@ I consider this a feature :-)
 
 =item * working locking
 
-Locking is done by locking C<passwd.lock> file.
+Locking is done by locking C<passwd> file.
 
 =back
 
@@ -3029,29 +3047,13 @@ L<Setup::Unix::User> and L<Setup::Unix::Group>, which use this module.
 
 L<Rinci>
 
-=head1 HOMEPAGE
-
-Please visit the project's homepage at L<https://metacpan.org/release/Unix-Passwd-File>.
-
-=head1 SOURCE
-
-Source repository is at L<https://github.com/perlancar/perl-Unix-Passwd-File>.
-
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Unix-Passwd-File>
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
 =head1 AUTHOR
 
 perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by perlancar@cpan.org.
+This software is copyright (c) 2016 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
